@@ -7,9 +7,19 @@ try:
 except ImportError:
     import configparser
 
+kamcli_formats_list = ["raw", "json", "table", "dict", "yaml"]
+
+COMMAND_ALIASES = {
+    "acl": "group",
+    "rpc": "jsonrpc",
+    "subs": "subscriber",
+    "sht": "htable",
+}
+
+
 def read_global_config(config_paths):
     """Get config."""
-    parser = configparser.SafeConfigParser()
+    parser = configparser.ConfigParser()
     if config_paths:
         parser.read(config_paths)
     else:
@@ -25,43 +35,38 @@ def read_global_config(config_paths):
 
 def parse_user_spec(ctx, ustr):
     """Get details of the user from ustr (username, aor or sip uri)"""
-    udata = { }
+    udata = {}
     if ":" in ustr:
         uaor = ustr.split(":")[1]
     else:
         uaor = ustr
     if "@" in uaor:
-        udata['username'] = uaor.split("@")[0]
-        udata['domain'] = uaor.split("@")[1]
+        udata["username"] = uaor.split("@")[0]
+        udata["domain"] = uaor.split("@")[1]
     else:
-        udata['username'] = uaor.split("@")[0]
+        udata["username"] = uaor.split("@")[0]
         try:
-            udata['domain'] = ctx.gconfig.get('main', 'domain')
+            udata["domain"] = ctx.gconfig.get("main", "domain")
         except configparser.NoOptionError:
             ctx.log("Default domain not set in config file")
             sys.exit()
-    if udata['username'] is None:
+    if udata["username"] is None:
         ctx.log("Failed to get username")
         sys.exit()
-    if udata['domain'] is None:
+    if udata["domain"] is None:
         ctx.log("Failed to get domain")
         sys.exit()
-    udata['username'] = udata['username'].encode('ascii','ignore').decode()
-    udata['domain'] = udata['domain'].encode('ascii','ignore').decode()
+    udata["username"] = udata["username"].encode("ascii", "ignore").decode()
+    udata["domain"] = udata["domain"].encode("ascii", "ignore").decode()
     return udata
 
 
+CONTEXT_SETTINGS = dict(
+    auto_envvar_prefix="KAMCLI", help_option_names=["-h", "--help"]
+)
 
 
-CONTEXT_SETTINGS = dict(auto_envvar_prefix='KAMCLI')
-
-COMMAND_ALIASES = {
-    "subs": "subscriber",
-    "rpc":  "jsonrpc",
-}
-
 class Context(object):
-
     def __init__(self):
         self.debug = False
         self.wdir = os.getcwd()
@@ -87,6 +92,18 @@ class Context(object):
             msg %= args
         click.echo(msg)
 
+    def printnlf(self, msg, *args):
+        """Print a formated message to stdout without new line."""
+        if args:
+            msg %= args
+        click.echo(msg, nl=False)
+
+    def read_config(self):
+        if self._gconfig is None:
+            self._gconfig = read_global_config(self.gconfig_paths)
+            if "cmdaliases" in self._gconfig:
+                COMMAND_ALIASES.update(self._gconfig["cmdaliases"])
+
     @property
     def gconfig(self):
         if self._gconfig is None:
@@ -95,16 +112,16 @@ class Context(object):
 
 
 pass_context = click.make_pass_decorator(Context, ensure=True)
-cmd_folder = os.path.abspath(os.path.join(os.path.dirname(__file__),
-                                          'commands'))
+cmd_folder = os.path.abspath(
+    os.path.join(os.path.dirname(__file__), "commands")
+)
+
 
 class KamCLI(click.MultiCommand):
-
     def list_commands(self, ctx):
         rv = []
         for filename in os.listdir(cmd_folder):
-            if filename.endswith('.py') and \
-               filename.startswith('cmd_'):
+            if filename.endswith(".py") and filename.startswith("cmd_"):
                 rv.append(filename[4:-3])
         rv.sort()
         return rv
@@ -114,40 +131,51 @@ class KamCLI(click.MultiCommand):
             name = COMMAND_ALIASES[name]
         try:
             if sys.version_info[0] == 2:
-                name = name.encode('ascii', 'replace')
-            mod = __import__('kamcli.commands.cmd_' + name,
-                             None, None, ['cli'])
+                name = name.encode("ascii", "replace")
+            mod = __import__(
+                "kamcli.commands.cmd_" + name, None, None, ["cli"]
+            )
         except ImportError:
             return
         return mod.cli
 
 
-def global_read_config(ctx, param, value):
-    """Callback that is used whenever --config is passed.  We use this to
-    always load the correct config.  This means that the config is loaded
-    even if the group itself never executes so our aliases stay always
-    available.
-    """
-    if value is None:
-        value = os.path.join(os.path.dirname(__file__), 'kamcli.ini')
-    ctx.read_config(value)
-    return value
-
-
-@click.command(cls=KamCLI, context_settings=CONTEXT_SETTINGS,
-                short_help='Kamailio command line interface control tool')
-@click.option('--wdir', type=click.Path(exists=True, file_okay=False,
-                                        resolve_path=True),
-              help='Working directory.')
-@click.option('--debug', '-d', is_flag=True,
-              help='Enable debug mode.')
-@click.option('--config', '-c',
-              default=None, help="Configuration file.")
-@click.option('nodefaultconfigs', '--no-default-configs', is_flag=True,
-            help='Skip loading default configuration files.')
+@click.command(
+    cls=KamCLI,
+    context_settings=CONTEXT_SETTINGS,
+    short_help="Kamailio command line interface control tool",
+)
+@click.option(
+    "-d", "--debug", "debug", is_flag=True, help="Enable debug mode."
+)
+@click.option(
+    "-c", "--config", "config", default=None, help="Configuration file."
+)
+@click.option(
+    "-w",
+    "--wdir",
+    "wdir",
+    type=click.Path(exists=True, file_okay=False, resolve_path=True),
+    help="Working directory.",
+)
+@click.option(
+    "-n",
+    "--no-default-configs",
+    "nodefaultconfigs",
+    is_flag=True,
+    help="Skip loading default configuration files.",
+)
+@click.option(
+    "-F",
+    "--output-format",
+    "oformat",
+    type=click.Choice(kamcli_formats_list),
+    default=None,
+    help="Format the output (overwriting db/jsonrpc outformat config attributes)",
+)
 @click.version_option()
 @pass_context
-def cli(ctx, debug, wdir, config, nodefaultconfigs):
+def cli(ctx, debug, config, wdir, nodefaultconfigs, oformat):
     """Kamailio command line interface control tool.
 
     \b
@@ -156,6 +184,8 @@ def cli(ctx, debug, wdir, config, nodefaultconfigs):
     \b
     Default configuration files:
         - /etc/kamcli/kamcli.ini
+        - ./kamcli.ini
+        - ./kamcli/kamcli.ini
         - ~/.kamcli/kamctli.ini
     Configs loading order: default configs, then --config option
 
@@ -167,6 +197,10 @@ def cli(ctx, debug, wdir, config, nodefaultconfigs):
     if wdir is not None:
         ctx.wdir = wdir
     if not nodefaultconfigs:
+        if os.path.isfile("./kamcli/kamcli.ini"):
+            ctx.gconfig_paths.append("./kamcli/kamcli.ini")
+        if os.path.isfile("./kamcli.ini"):
+            ctx.gconfig_paths.append("./kamcli.ini")
         if os.path.isfile("/etc/kamcli/kamcli.ini"):
             ctx.gconfig_paths.append("/etc/kamcli/kamcli.ini")
         tpath = os.path.expanduser("~/.kamcli/kamcli.ini")
@@ -174,4 +208,7 @@ def cli(ctx, debug, wdir, config, nodefaultconfigs):
             ctx.gconfig_paths.append(tpath)
     if config is not None:
         ctx.gconfig_paths.append(os.path.expanduser(config))
-
+    ctx.read_config()
+    if oformat is not None:
+        ctx.gconfig.set("db", "outformat", oformat)
+        ctx.gconfig.set("jsonrpc", "outformat", oformat)
