@@ -1,14 +1,11 @@
-import os
-import time
 import click
 from kamcli.cli import pass_context
 from kamcli.iorpc import command_ctl
 
 
-##
-#
-#
-@click.command('rpcmethods', short_help='Print the list of available raw RPC methods')
+@click.command(
+    "rpcmethods", short_help="Print the list of available raw RPC methods"
+)
 @pass_context
 def cli(ctx):
     """Print the list of available raw RPC methods
@@ -19,4 +16,4 @@ def cli(ctx):
         - kamcli rpcmethods
         - kamcli rpc <method> <params>
     """
-    command_ctl(ctx, 'system.listMethods', [])
+    command_ctl(ctx, "system.listMethods", [])
